import fs from 'fs';
import { dirname, resolve } from 'path';
import glob from 'glob';
import showdown from 'showdown';

export default build;

const CONFIGS_LOOKUP_PATTERN = '+(common|components|docs)/**/stories/**/*.config.+(js|json)';
const DEFAULT_OUTPUT_FILE_SUFFIX = '.autogenerated.stories.js';

const FS_OPTIONS = { encoding: 'utf-8' };
const converter = new showdown.Converter();
converter.setFlavor('github');

function build() {
	const configPaths = collectConfigurations();
	for (const configPath of configPaths) {
		if (configPath.endsWith('js')) {
			processJsConfiguration(configPath);
		} else {
			processJsonConfiguration(configPath);
		}
	}
}

function collectConfigurations() {
	console.info('collecting configurations...');
	const result = glob.sync(CONFIGS_LOOKUP_PATTERN);
	console.info(`... ${result.length} configuration/s collected`);
	return result;
}

function processJsConfiguration(configPath) {
	throw new Error(`JS configuration flavor is not yet supported; configuration path: '${configPath}'`);
}

function processJsonConfiguration(configPath) {
	console.info(`processing '${configPath}'...`);

	//	config
	const config = JSON.parse(fs.readFileSync(configPath, FS_OPTIONS));
	validateConfig(config);

	//	convert
	const configDir = dirname(configPath);
	const mdInput = fs.readFileSync(resolve(configDir, config.sourcePath), FS_OPTIONS);
	const htmlOut = converter.makeHtml(mdInput);
	const storyJs = buildStoryJs(config.story, htmlOut);

	//	dump
	const outputFileName = config.outputName + (config.outputName.endsWith('.js')
		? ''
		: DEFAULT_OUTPUT_FILE_SUFFIX);
	fs.writeFileSync(resolve(configDir, outputFileName), storyJs, FS_OPTIONS);

	console.info('... done');
}

function validateConfig(config) {
	if (!config || typeof config !== 'object') {
		throw new Error(`config MUST be a non-null object; violator: ${config}`);
	}
	if (!config.sourcePath || typeof config.sourcePath !== 'string') {
		throw new Error(`'sourcePath' MUST be a non-empty string; got '${config.sourcePath}'`);
	}
	if (!config.outputName || typeof config.outputName !== 'string') {
		throw new Error(`'outputPath' MUST be a non-empty string; got '${config.outputName}'`);
	}
	if (!config.story || typeof config.story !== 'object') {
		throw new Error(`'story' MUST be a non-null object; got '${config.story}'`);
	}
	if (!config.story.title || typeof config.story.title !== 'string') {
		throw new Error(`'story.title' MUST be a non-empty string; got '${config.story.title}'`);
	}
	if (!config.story.name || typeof config.story.name !== 'string') {
		throw new Error(`'story.name' MUST be a non-empty string; got '${config.story.name}'`);
	}
}

function buildStoryJs(story, html) {
	let result = `
	//	Autogenerated at ${new Date().toISOString()}
	//
	export default {
		title: '${story.title}'
	};

	export const ${story.name} = () => \`${html}\`;
	`;

	if (story.parameters) {
		result += `
	${story.name}.parameters = ${JSON.stringify(story.parameters)};
		`;
	}

	return result;
}